//Published by Ryan Morrison 7-16-17
//This sketch reads in the data of four backup sensors (A,B,C, and D) 
//from the transceiver and displays the data on the serial monitor. 

//Connect blue data wire to pin 32 (or change this if you want another one) (any digital pin will do)
int pin = 12;
//Teensy LED pin is number 13 (used to show that data is being read)
int ledPin = 13;

//values of the four sensors as well as the minimum value out of the four
float Avalue,Bvalue,Cvalue,Dvalue,minimum_value;
//boolean variable that is used to detect start bit
boolean high_detected = false;
//boolean variable set to true whenever one of the sensors is less than 0.34 meters
boolean e_activated = false;





void setup(){
  pinMode(pin, INPUT);
  pinMode(ledPin, OUTPUT);
  Serial.begin(115200);
}



void loop(){
  //input 0 for meters, 1 for feet
  update_backup_sensor_values(1);
}



void update_backup_sensor_values(int meter0_or_feet_1){
  byte address;
  byte value;
  byte pulse_value;
  float current_value_for_unit_selected;
  float meter_value;
  float feet_value;
  unsigned long pulse_length;
  byte sensorValue[24];

  //look for starter pulse
  pulse_length = pulseIn(pin, HIGH);
  if(pulse_length > 500){
    int i;
    //read in the next 17 values
    for (i = 0; i < 17; i = i + 1) {
      pulse_length = pulseIn(pin, HIGH);
      if (pulse_length > 20 && pulse_length < 120) {
        pulse_value = B0; //value 0
        digitalWrite(ledPin, LOW);
      }
    else{
      pulse_value = B1;  //value 1
      digitalWrite(ledPin, HIGH);
    }
    sensorValue[i] = pulse_value;
    }
  
  //  //print the all the bits 
  //  for (i = 0; i < 19; i = i + 1) {
  //    Serial.print(sensorValue[i], BIN);
  //  }
  //  Serial.println(" ");
  
    //determine if A,B,C, or D sensor and calculate value
    address = (sensorValue[6] << 2) + (sensorValue[7] << 1) + sensorValue[8];
    //Address 0 and 1 = Sensor A
    //Address 2 and 3 = Sensor B
    //Address 4 and 5 = Sensor C
    //Address 6 and 7 = Sensor D
    value = (sensorValue[9] << 7) + (sensorValue[10] << 6) + (sensorValue[11] << 5) + (sensorValue[12] << 4) + (sensorValue[13] << 3) + (sensorValue[14] << 2) + (sensorValue[15] << 1) + sensorValue[16];
    
    //Convert to meters 
    //not sure why the output of the value variable is not exactly meters, but these next two steps are necessary
    //to output exactly what is on the LCD of the included display
    if(address == 0 || address == 2 || address == 4 || address == 6){
      meter_value = float(value)*0.005;
    }
    else if(address == 1 || address == 3 || address == 5 || address == 7){
      meter_value = (float(value)+256)*0.005;
    }

    //determine if out of range
    if(value == 254){
      meter_value = 999;
      feet_value = 999;
    }

    //checks if user wants feet or meters and converts to feet if necessary
    if(meter0_or_feet_1 == 0){
      current_value_for_unit_selected = meter_value;
    }
    else{
      feet_value = meter_value*3.28084;
      current_value_for_unit_selected = feet_value;
    }
    
    //Assigns calculated value to certain sensor based off of address
    if(address == 0 || address == 1){
      Serial.print("A: ");
      Avalue = current_value_for_unit_selected;
      Serial.println(Avalue);
    }
    else if(address == 2 || address == 3){
      //Serial.print("B: ");
      Bvalue = current_value_for_unit_selected;
      //Serial.println(Bvalue);
    }
    else if(address == 4 || address == 5){
      //Serial.print("C: ");
      Cvalue = current_value_for_unit_selected;
      //Serial.println(Cvalue);
    }
    else if (address == 6 || address == 7){
      //Serial.print("D: ");
      Dvalue = current_value_for_unit_selected;
      //Serial.println(Dvalue);
    }
    else{
      //Serial.print("Other address: ");
      //Serial.print(address);
      //Serial.print(" ");
      //Serial.println(value);
    }

    //determine the minimum value
    minimum_value = min(Avalue,min(Bvalue,min(Cvalue,Dvalue)));
    if((minimum_value < 0.34 && meter0_or_feet_1 == 0) || (minimum_value < (0.34*3.28084) && meter0_or_feet_1 == 1)){
      //this stands for emergency I am pretty sure. It was a small "E" icon on the included LCD display
      e_activated = true;
    }
    if(e_activated){
      //Serial.print(minimum_value);
      //Serial.println(" E");
      e_activated = false;
    }
    else{
      //Serial.println(minimum_value);
    }
  }
}

